PRINT ''
GO
PRINT '*** Creating Error Log objects'
GO
CREATE TABLE [dbo].[ErrorLog] (
    [ErrorLogID]     INT             IDENTITY (1, 1) NOT NULL,
    [ErrorTime]      DATETIME        CONSTRAINT [DF_ErrorLog_ErrorTime] DEFAULT (GETDATE()) NOT NULL,
    [UserName]       [sysname]       NOT NULL,
    [ErrorNumber]    INT             NOT NULL,
    [ErrorSeverity]  INT             NULL,
    [ErrorState]     INT             NULL,
    [ErrorProcedure] NVARCHAR (126)  NULL,
    [ErrorLine]      INT             NULL,
    [ErrorMessage]   NVARCHAR (4000) NOT NULL
)
GO
ALTER TABLE [dbo].[ErrorLog] WITH CHECK
    ADD CONSTRAINT [PK_ErrorLog_ErrorLogID] PRIMARY KEY CLUSTERED ([ErrorLogID])
GO
CREATE PROCEDURE [dbo].[uspPrintError]

AS
BEGIN
    SET NOCOUNT ON;
    PRINT 'Error ' + CONVERT (VARCHAR (50), ERROR_NUMBER()) + ', Severity ' + CONVERT (VARCHAR (5), ERROR_SEVERITY()) + ', State ' + CONVERT (VARCHAR (5), ERROR_STATE()) + ', Procedure ' + ISNULL(ERROR_PROCEDURE(), '-') + ', Line ' + CONVERT (VARCHAR (5), ERROR_LINE());
    PRINT ERROR_MESSAGE();
END
GO
CREATE PROCEDURE [dbo].[uspLogError]
@ErrorLogID INT=0 OUTPUT
AS
BEGIN
    SET NOCOUNT ON;
    SET @ErrorLogID = 0;
    BEGIN TRY
        IF ERROR_NUMBER() IS NULL
            RETURN;
        IF XACT_STATE() = -1
            BEGIN
                PRINT 'Cannot log error since the current transaction is in an uncommittable state. ' + 'Rollback the transaction before executing uspLogError in order to successfully log error information.';
                RETURN;
            END
        INSERT  [dbo].[ErrorLog] ([UserName], [ErrorNumber], [ErrorSeverity], [ErrorState], [ErrorProcedure], [ErrorLine], [ErrorMessage])
        VALUES                  (CONVERT (sysname, CURRENT_USER), ERROR_NUMBER(), ERROR_SEVERITY(), ERROR_STATE(), ERROR_PROCEDURE(), ERROR_LINE(), ERROR_MESSAGE());
        SET @ErrorLogID = @@IDENTITY;
    END TRY
    BEGIN CATCH
        PRINT 'An error occurred in stored procedure uspLogError: ';
        EXECUTE [dbo].[uspPrintError] ;
        RETURN -1;
    END CATCH
END
GO
PRINT ''
GO
PRINT '*** Creating Data Types'
GO
CREATE TYPE [AccountNumber]
    FROM NVARCHAR (15) NULL
GO
CREATE TYPE [Flag]
    FROM BIT NOT NULL
GO
CREATE TYPE [NameStyle]
    FROM BIT NOT NULL
GO
CREATE TYPE [Name]
    FROM NVARCHAR (50) NULL
GO
CREATE TYPE [OrderNumber]
    FROM NVARCHAR (25) NULL
GO
CREATE TYPE [Phone]
    FROM NVARCHAR (25) NULL
GO
PRINT ''
GO
PRINT '*** Creating Database Schemas'
GO
CREATE SCHEMA [SalesLT]
    AUTHORIZATION [dbo]
GO
PRINT ''
GO
PRINT '*** Creating XML Schemas'
GO
PRINT ''
GO
PRINT 'Create Product catalog description schema'
GO
PRINT ''
GO
PRINT '*** Creating Tables'
GO
CREATE TABLE [dbo].[BuildVersion] (
    [SystemInformationID] TINYINT       IDENTITY (1, 1) NOT NULL,
    [Database Version]    NVARCHAR (25) NOT NULL,
    [VersionDate]         DATETIME      NOT NULL,
    [ModifiedDate]        DATETIME      CONSTRAINT [DF_BuildVersion_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[Address] (
    [AddressID]     INT              IDENTITY (1, 1) NOT NULL,
    [AddressLine1]  NVARCHAR (60)    NOT NULL,
    [AddressLine2]  NVARCHAR (60)    NULL,
    [City]          NVARCHAR (30)    NOT NULL,
    [StateProvince] [Name]           NOT NULL,
    [CountryRegion] [Name]           NOT NULL,
    [PostalCode]    NVARCHAR (15)    NOT NULL,
    [rowguid]       UNIQUEIDENTIFIER CONSTRAINT [DF_Address_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]  DATETIME         CONSTRAINT [DF_Address_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[Customer] (
    [CustomerID]   INT              IDENTITY (1, 1) NOT NULL,
    [NameStyle]    [NameStyle]      CONSTRAINT [DF_Customer_NameStyle] DEFAULT (0) NOT NULL,
    [Title]        NVARCHAR (8)     NULL,
    [FirstName]    [Name]           NOT NULL,
    [MiddleName]   [Name]           NULL,
    [LastName]     [Name]           NOT NULL,
    [Suffix]       NVARCHAR (10)    NULL,
    [CompanyName]  NVARCHAR (128)   NULL,
    [SalesPerson]  NVARCHAR (256)   NULL,
    [EmailAddress] NVARCHAR (50)    NULL,
    [Phone]        [Phone]          NULL,
    [PasswordHash] VARCHAR (128)    NOT NULL,
    [PasswordSalt] VARCHAR (10)     NOT NULL,
    [rowguid]      UNIQUEIDENTIFIER CONSTRAINT [DF_Customer_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate] DATETIME         CONSTRAINT [DF_Customer_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[CustomerAddress] (
    [CustomerID]   INT              NOT NULL,
    [AddressID]    INT              NOT NULL,
    [AddressType]  [Name]           NOT NULL,
    [rowguid]      UNIQUEIDENTIFIER CONSTRAINT [DF_CustomerAddress_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate] DATETIME         CONSTRAINT [DF_CustomerAddress_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[Product] (
    [ProductID]              INT              IDENTITY (1, 1) NOT NULL,
    [Name]                   [Name]           NOT NULL,
    [ProductNumber]          NVARCHAR (25)    NOT NULL,
    [Color]                  NVARCHAR (15)    NULL,
    [StandardCost]           MONEY            NOT NULL,
    [ListPrice]              MONEY            NOT NULL,
    [Size]                   NVARCHAR (5)     NULL,
    [Weight]                 DECIMAL (8, 2)   NULL,
    [ProductCategoryID]      INT              NULL,
    [ProductModelID]         INT              NULL,
    [SellStartDate]          DATETIME         NOT NULL,
    [SellEndDate]            DATETIME         NULL,
    [DiscontinuedDate]       DATETIME         NULL,
    [ThumbNailPhoto]         VARBINARY (MAX)  NULL,
    [ThumbnailPhotoFileName] NVARCHAR (50)    NULL,
    [rowguid]                UNIQUEIDENTIFIER CONSTRAINT [DF_Product_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]           DATETIME         CONSTRAINT [DF_Product_ModifiedDate] DEFAULT (GETDATE()) NOT NULL,
    CONSTRAINT [CK_Product_StandardCost] CHECK ([StandardCost] >= 0.00),
    CONSTRAINT [CK_Product_ListPrice] CHECK ([ListPrice] >= 0.00),
    CONSTRAINT [CK_Product_Weight] CHECK ([Weight] > 0.00),
    CONSTRAINT [CK_Product_SellEndDate] CHECK (([SellEndDate] >= [SellStartDate])
                                               OR ([SellEndDate] IS NULL))
)
GO
CREATE TABLE [SalesLT].[ProductCategory] (
    [ProductCategoryID]       INT              IDENTITY (1, 1) NOT NULL,
    [ParentProductCategoryID] INT              NULL,
    [Name]                    [Name]           NOT NULL,
    [rowguid]                 UNIQUEIDENTIFIER CONSTRAINT [DF_ProductCategory_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]            DATETIME         CONSTRAINT [DF_ProductCategory_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[ProductDescription] (
    [ProductDescriptionID] INT              IDENTITY (1, 1) NOT NULL,
    [Description]          NVARCHAR (400)   NOT NULL,
    [rowguid]              UNIQUEIDENTIFIER CONSTRAINT [DF_ProductDescription_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]         DATETIME         CONSTRAINT [DF_ProductDescription_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[ProductModel] (
    [ProductModelID]     INT              IDENTITY (1, 1) NOT NULL,
    [Name]               [Name]           NOT NULL,
    [CatalogDescription] XML              NULL,
    [rowguid]            UNIQUEIDENTIFIER CONSTRAINT [DF_ProductModel_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]       DATETIME         CONSTRAINT [DF_ProductModel_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[ProductModelProductDescription] (
    [ProductModelID]       INT              NOT NULL,
    [ProductDescriptionID] INT              NOT NULL,
    [Culture]              NCHAR (6)        NOT NULL,
    [rowguid]              UNIQUEIDENTIFIER CONSTRAINT [DF_ProductModelProductDescription_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]         DATETIME         CONSTRAINT [DF_ProductModelProductDescription_ModifiedDate] DEFAULT (GETDATE()) NOT NULL
)
GO
CREATE TABLE [SalesLT].[SalesOrderDetail] (
    [SalesOrderID]       INT              NOT NULL,
    [SalesOrderDetailID] INT              IDENTITY (1, 1) NOT NULL,
    [OrderQty]           SMALLINT         NOT NULL,
    [ProductID]          INT              NOT NULL,
    [UnitPrice]          MONEY            NOT NULL,
    [UnitPriceDiscount]  MONEY            CONSTRAINT [DF_SalesOrderDetail_UnitPriceDiscount] DEFAULT (0.0) NOT NULL,
    [LineTotal]          AS               ISNULL([UnitPrice] * (1.0 - [UnitPriceDiscount]) * [OrderQty], 0.0),
    [rowguid]            UNIQUEIDENTIFIER CONSTRAINT [DF_SalesOrderDetail_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]       DATETIME         CONSTRAINT [DF_SalesOrderDetail_ModifiedDate] DEFAULT (GETDATE()) NOT NULL,
    CONSTRAINT [CK_SalesOrderDetail_OrderQty] CHECK ([OrderQty] > 0),
    CONSTRAINT [CK_SalesOrderDetail_UnitPrice] CHECK ([UnitPrice] >= 0.00),
    CONSTRAINT [CK_SalesOrderDetail_UnitPriceDiscount] CHECK ([UnitPriceDiscount] >= 0.00)
)
GO
CREATE TABLE [SalesLT].[SalesOrderHeader] (
    [SalesOrderID]           INT              IDENTITY (1, 1) NOT NULL,
    [RevisionNumber]         TINYINT          CONSTRAINT [DF_SalesOrderHeader_RevisionNumber] DEFAULT (0) NOT NULL,
    [OrderDate]              DATETIME         CONSTRAINT [DF_SalesOrderHeader_OrderDate] DEFAULT (GETDATE()) NOT NULL,
    [DueDate]                DATETIME         NOT NULL,
    [ShipDate]               DATETIME         NULL,
    [Status]                 TINYINT          CONSTRAINT [DF_SalesOrderHeader_Status] DEFAULT (1) NOT NULL,
    [OnlineOrderFlag]        [Flag]           CONSTRAINT [DF_SalesOrderHeader_OnlineOrderFlag] DEFAULT (1) NOT NULL,
    [SalesOrderNumber]       AS               ISNULL(N'SO' + CONVERT (NVARCHAR (23), [SalesOrderID]), N'*** ERROR ***'),
    [PurchaseOrderNumber]    [OrderNumber]    NULL,
    [AccountNumber]          [AccountNumber]  NULL,
    [CustomerID]             INT              NOT NULL,
    [ShipToAddressID]        INT              NULL,
    [BillToAddressID]        INT              NULL,
    [ShipMethod]             NVARCHAR (50)    NOT NULL,
    [CreditCardApprovalCode] VARCHAR (15)     NULL,
    [SubTotal]               MONEY            CONSTRAINT [DF_SalesOrderHeader_SubTotal] DEFAULT (0.00) NOT NULL,
    [TaxAmt]                 MONEY            CONSTRAINT [DF_SalesOrderHeader_TaxAmt] DEFAULT (0.00) NOT NULL,
    [Freight]                MONEY            CONSTRAINT [DF_SalesOrderHeader_Freight] DEFAULT (0.00) NOT NULL,
    [TotalDue]               AS               ISNULL([SubTotal] + [TaxAmt] + [Freight], 0),
    [Comment]                NVARCHAR (MAX)   NULL,
    [rowguid]                UNIQUEIDENTIFIER CONSTRAINT [DF_SalesOrderHeader_rowguid] DEFAULT (NEWID()) NOT NULL,
    [ModifiedDate]           DATETIME         CONSTRAINT [DF_SalesOrderHeader_ModifiedDate] DEFAULT (GETDATE()) NOT NULL,
    CONSTRAINT [CK_SalesOrderHeader_Status] CHECK ([Status] BETWEEN 0 AND 8),
    CONSTRAINT [CK_SalesOrderHeader_DueDate] CHECK ([DueDate] >= [OrderDate]),
    CONSTRAINT [CK_SalesOrderHeader_ShipDate] CHECK (([ShipDate] >= [OrderDate])
                                                     OR ([ShipDate] IS NULL)),
    CONSTRAINT [CK_SalesOrderHeader_SubTotal] CHECK ([SubTotal] >= 0.00),
    CONSTRAINT [CK_SalesOrderHeader_TaxAmt] CHECK ([TaxAmt] >= 0.00),
    CONSTRAINT [CK_SalesOrderHeader_Freight] CHECK ([Freight] >= 0.00)
)
GO
PRINT ''
GO
PRINT '*** Loading Data'
GO
PRINT 'Loading [dbo].[BuildVersion]'
GO
PRINT 'Loading [SalesLT].[Address]'
GO
PRINT 'Loading [SalesLT].[Customer]'
GO
PRINT 'Loading [SalesLT].[CustomerAddress]'
GO
PRINT 'Loading [SalesLT].[Product]'
GO
PRINT 'Loading [SalesLT].[ProductCategory]'
GO
PRINT 'Loading [SalesLT].[ProductDescription]'
GO
PRINT 'Loading [SalesLT].[ProductModel]'
GO
PRINT 'Loading [SalesLT].[ProductModelProductDescription]'
GO
PRINT 'Loading [SalesLT].[SalesOrderDetail]'
GO
PRINT 'Loading [SalesLT].[SalesOrderHeader]'
GO
PRINT ''
GO
PRINT '*** Adding Primary Keys'
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER TABLE [SalesLT].[Address] WITH CHECK
    ADD CONSTRAINT [PK_Address_AddressID] PRIMARY KEY CLUSTERED ([AddressID])
GO
ALTER TABLE [SalesLT].[Customer] WITH CHECK
    ADD CONSTRAINT [PK_Customer_CustomerID] PRIMARY KEY CLUSTERED ([CustomerID])
GO
ALTER TABLE [SalesLT].[CustomerAddress] WITH CHECK
    ADD CONSTRAINT [PK_CustomerAddress_CustomerID_AddressID] PRIMARY KEY CLUSTERED ([CustomerID], [AddressID])
GO
ALTER TABLE [SalesLT].[Product] WITH CHECK
    ADD CONSTRAINT [PK_Product_ProductID] PRIMARY KEY CLUSTERED ([ProductID])
GO
ALTER TABLE [SalesLT].[ProductCategory] WITH CHECK
    ADD CONSTRAINT [PK_ProductCategory_ProductCategoryID] PRIMARY KEY CLUSTERED ([ProductCategoryID])
GO
ALTER TABLE [SalesLT].[ProductDescription] WITH CHECK
    ADD CONSTRAINT [PK_ProductDescription_ProductDescriptionID] PRIMARY KEY CLUSTERED ([ProductDescriptionID])
GO
ALTER TABLE [SalesLT].[ProductModel] WITH CHECK
    ADD CONSTRAINT [PK_ProductModel_ProductModelID] PRIMARY KEY CLUSTERED ([ProductModelID])
GO
ALTER TABLE [SalesLT].[ProductModelProductDescription] WITH CHECK
    ADD CONSTRAINT [PK_ProductModelProductDescription_ProductModelID_ProductDescriptionID_Culture] PRIMARY KEY CLUSTERED ([ProductModelID], [ProductDescriptionID], [Culture])
GO
ALTER TABLE [SalesLT].[SalesOrderDetail] WITH CHECK
    ADD CONSTRAINT [PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID] PRIMARY KEY CLUSTERED ([SalesOrderID], [SalesOrderDetailID])
GO
ALTER TABLE [SalesLT].[SalesOrderHeader] WITH CHECK
    ADD CONSTRAINT [PK_SalesOrderHeader_SalesOrderID] PRIMARY KEY CLUSTERED ([SalesOrderID])
GO
PRINT ''
GO
PRINT '*** Adding Unique Constraints'
GO
ALTER TABLE [SalesLT].[Address] WITH CHECK
    ADD CONSTRAINT [AK_Address_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[Customer] WITH CHECK
    ADD CONSTRAINT [AK_Customer_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[CustomerAddress] WITH CHECK
    ADD CONSTRAINT [AK_CustomerAddress_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[Product] WITH CHECK
    ADD CONSTRAINT [AK_Product_ProductNumber] UNIQUE ([ProductNumber]),
        CONSTRAINT [AK_Product_Name] UNIQUE ([Name]),
        CONSTRAINT [AK_Product_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[ProductCategory] WITH CHECK
    ADD CONSTRAINT [AK_ProductCategory_Name] UNIQUE ([Name]),
        CONSTRAINT [AK_ProductCategory_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[ProductDescription] WITH CHECK
    ADD CONSTRAINT [AK_ProductDescription_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[ProductModel] WITH CHECK
    ADD CONSTRAINT [AK_ProductModel_Name] UNIQUE ([Name]),
        CONSTRAINT [AK_ProductModel_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[ProductModelProductDescription] WITH CHECK
    ADD CONSTRAINT [AK_ProductModelProductDescription_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[SalesOrderDetail] WITH CHECK
    ADD CONSTRAINT [AK_SalesOrderDetail_rowguid] UNIQUE ([rowguid])
GO
ALTER TABLE [SalesLT].[SalesOrderHeader] WITH CHECK
    ADD CONSTRAINT [AK_SalesOrderHeader_SalesOrderNumber] UNIQUE ([SalesOrderNumber]),
        CONSTRAINT [AK_SalesOrderHeader_rowguid] UNIQUE ([rowguid])
GO
PRINT ''
GO
PRINT '*** Adding Indexes'
GO
CREATE INDEX [IX_Address_AddressLine1_AddressLine2_City_StateProvince_PostalCode_CountryRegion]
    ON [SalesLT].[Address]([AddressLine1], [AddressLine2], [City], [StateProvince], [PostalCode], [CountryRegion])
GO
CREATE INDEX [IX_Address_StateProvince]
    ON [SalesLT].[Address]([StateProvince])
GO
CREATE INDEX [IX_Customer_EmailAddress]
    ON [SalesLT].[Customer]([EmailAddress])
GO
CREATE INDEX [IX_SalesOrderDetail_ProductID]
    ON [SalesLT].[SalesOrderDetail]([ProductID])
GO
CREATE INDEX [IX_SalesOrderHeader_CustomerID]
    ON [SalesLT].[SalesOrderHeader]([CustomerID])
GO
PRINT ''
GO
PRINT '*** Creating an XML index for the XML column'
GO
SET ARITHABORT ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO
SET ANSI_WARNINGS ON
GO
SET CONCAT_NULL_YIELDS_NULL ON
GO
SET NUMERIC_ROUNDABORT OFF
GO
PRINT ''
GO
PRINT '*** Creating Foreign Key Constraints'
GO
ALTER TABLE [dbo].[BuildVersion]
    ADD CONSTRAINT [PK_BuildVersion_SystemInformationID] PRIMARY KEY CLUSTERED ([SystemInformationID])
GO
